@@ -1,6 +1,6 @@
 #
 #  Copyright (C) 2004-2016 Friedrich Leisch and Bettina Gruen
-#  $Id: flexmix.R 5115 2017-04-07 08:18:13Z gruen $
+#  $Id: flexmix.R 5184 2020-06-20 18:27:29Z gruen $
 #
 
 log_row_sums <- function(m) {
@@ -365,13 +365,11 @@ groupFirst <- function(x) !duplicated(x)
 ## if we have a group variable, set the posterior to the product
 ## of all density values for that group (=sum in logarithm)
 groupPosteriors <- function(x, group)
-{    
-    for(g in levels(group)){
-        gok <- group==g
-        if(any(gok)){
-            x[gok,] <- matrix(colSums(x[gok,,drop=FALSE]),
-                              nrow=sum(gok), ncol=ncol(x), byrow=TRUE)
-        }
+{
+    if (length(group) > 0) {
+        group <- as.integer(group)
+        x.by.group <- matrix(unname(apply(x, 2, tapply, group, sum)), ncol = ncol(x))
+        x <- x.by.group[group,, drop = FALSE]
     }
     x
 }
@@ -496,7 +494,7 @@ function(object, newdata, ...) {
     groups <- .FLXgetGrouping(object@formula, newdata)
     nobs <- if (is(newdata, "data.frame")) nrow(newdata)
             else min(sapply(newdata, function(x) {
-              if (is(x, "matrix")) nrow(x) else length(x)
+              if (is.null(nrow(x))) length(x) else nrow(x)
             }))
     group <- if (length(groups$group)) groups$group else factor(seq_len(nobs))
     object@concomitant <- FLXgetModelmatrix(object@concomitant, data = newdata,
